"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.registerGetRoute = registerGetRoute;

var _queryString = require("query-string");

var _constants = require("../../../../../common/constants");

var _utils = require("../../../../lib/utils");

/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0 and the Server Side Public License, v 1; you may not use this file except
 * in compliance with, at your election, the Elastic License 2.0 or the Server
 * Side Public License, v 1.
 */
const RESPONSE_SIZE_LIMIT = 10 * 1024 * 1024; // Limit the response size to 10MB, because the response can be very large and sending it to the client
// can cause the browser to hang.

async function getMappings(esClient, settings) {
  if (settings.fields) {
    const stream = await esClient.asInternalUser.indices.getMapping(undefined, {
      asStream: true
    });
    return (0, _utils.streamToJSON)(stream, RESPONSE_SIZE_LIMIT);
  } // If the user doesn't want autocomplete suggestions, then clear any that exist.


  return {};
}

async function getAliases(esClient, settings) {
  if (settings.indices) {
    const stream = await esClient.asInternalUser.indices.getAlias(undefined, {
      asStream: true
    });
    return (0, _utils.streamToJSON)(stream, RESPONSE_SIZE_LIMIT);
  } // If the user doesn't want autocomplete suggestions, then clear any that exist.


  return {};
}

async function getDataStreams(esClient, settings) {
  if (settings.dataStreams) {
    const stream = await esClient.asInternalUser.indices.getDataStream(undefined, {
      asStream: true
    });
    return (0, _utils.streamToJSON)(stream, RESPONSE_SIZE_LIMIT);
  } // If the user doesn't want autocomplete suggestions, then clear any that exist.


  return {};
}

async function getLegacyTemplates(esClient, settings) {
  if (settings.templates) {
    const stream = await esClient.asInternalUser.indices.getTemplate(undefined, {
      asStream: true
    });
    return (0, _utils.streamToJSON)(stream, RESPONSE_SIZE_LIMIT);
  } // If the user doesn't want autocomplete suggestions, then clear any that exist.


  return {};
}

async function getComponentTemplates(esClient, settings) {
  if (settings.templates) {
    const stream = await esClient.asInternalUser.cluster.getComponentTemplate(undefined, {
      asStream: true
    });
    return (0, _utils.streamToJSON)(stream, RESPONSE_SIZE_LIMIT);
  } // If the user doesn't want autocomplete suggestions, then clear any that exist.


  return {};
}

async function getIndexTemplates(esClient, settings) {
  if (settings.templates) {
    const stream = await esClient.asInternalUser.indices.getIndexTemplate(undefined, {
      asStream: true
    });
    return (0, _utils.streamToJSON)(stream, RESPONSE_SIZE_LIMIT);
  } // If the user doesn't want autocomplete suggestions, then clear any that exist.


  return {};
}

function registerGetRoute({
  router,
  lib: {
    handleEsError
  }
}) {
  router.get({
    path: `${_constants.API_BASE_PATH}/autocomplete_entities`,
    validate: false
  }, async (ctx, request, response) => {
    try {
      const settings = (0, _queryString.parse)(request.url.search, {
        parseBooleans: true
      }); // If no settings are provided return 400

      if (Object.keys(settings).length === 0) {
        return response.badRequest({
          body: 'Request must contain a query param of autocomplete settings'
        });
      }

      const esClient = (await ctx.core).elasticsearch.client; // Wait for all requests to complete, in case one of them fails return the successfull ones

      const results = await Promise.allSettled([getMappings(esClient, settings), getAliases(esClient, settings), getDataStreams(esClient, settings), getLegacyTemplates(esClient, settings), getIndexTemplates(esClient, settings), getComponentTemplates(esClient, settings)]);
      const [mappings, aliases, dataStreams, legacyTemplates, indexTemplates, componentTemplates] = results.map(result => {
        // If the request was successful, return the result
        if (result.status === 'fulfilled') {
          return result.value;
        } // If the request failed, return an empty object


        return {};
      });
      return response.ok({
        body: {
          mappings,
          aliases,
          dataStreams,
          legacyTemplates,
          indexTemplates,
          componentTemplates
        }
      });
    } catch (e) {
      return handleEsError({
        error: e,
        response
      });
    }
  });
}